from tudelft.utilities.immutablelist.AbstractPermutations import AbstractPermutations
from typing import TypeVar, List, Tuple
from tudelft.utilities.immutablelist.ImmutableList import ImmutableList
from tudelft.utilities.immutablelist.ListWithRemove import ListWithRemove
from tudelft.utilities.immutablelist.FixedList import FixedList
from math import factorial

E = TypeVar('E')
class PermutationsWithoutReturn(AbstractPermutations[E]) :
	'''
	Creates all permutation without return (WOR). NOTICE: this uses
	ListWithRemove and therefore is limited in the number of removals that can be
	done.
	
	@param <E> type of the elements
	'''
	
	def __init__(self, lst:ImmutableList[E] , n:int ) :
		'''
		all permutations of a given list, drawing n items from the list
		@param list list to be permuted
		@param n    the number of items to draw from the list. Must be between 0
		*             and size of list.
		'''

		super().__init__(lst, n)
		
		if n> lst.size():
			raise ValueError("n bigger than list size")


	#Override
	def get(self, index:int)->ImmutableList[E] :
		'''
		Method: we need to draw #drawsize items. The first has #drawlistsize
		choices, the second #drawlistsize-1 etc. So we convert #index into
		#drawsize indices, the first running up to #drawlistsize, the second
		up to #drawlistsize-1, etc.
		'''
		sublist:ListWithRemove[E] = ListWithRemove(self._drawlist,set())
		element:List[E]  = []
		i:int = index;
		for item in range(self._drawsize):
			divrem:Tuple[int,int] = divmod(i, self._drawlistsize - item)
			im:int = divrem[1]
			element.append(sublist.get(im))
			sublist = sublist.remove(im)
			i = divrem[0]
		return FixedList(element)

	#Override
	def size(self) ->int:
		return factorial(self._drawlistsize) // factorial(self._drawlistsize - self._drawsize)

